/*
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * David Mueller, ELSOFT AG, <d.mueller@elsoft.ch>
 *
 * (C) Copyright 2003
 * Texas Instruments, <www.ti.com>
 * Kshitij Gupta <Kshitij@ti.com>
 *
 * (C) Copyright 2004
 * ARM Ltd.
 * Philippe Robin, <philippe.robin@arm.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <common.h>
#include <linux-adapter.h>
#include <asm/arch/platform.h>
#include <gpio_i2c.h>

void peripheral_power_enable (void);
void hisi_oem_init(void);

#if defined(CONFIG_SHOW_BOOT_PROGRESS)
void show_boot_progress(int progress)
{
    printf("Boot reached stage %d\n", progress);
}
#endif

#define COMP_MODE_ENABLE ((unsigned int)0x0000EAEF)

static inline void delay (unsigned long loops)
{
	__asm__ volatile ("1:\n"
		"subs %0, %1, #1\n"
		"bne 1b":"=r" (loops):"0" (loops));
}

/*
 * Miscellaneous platform dependent initialisations
 */
int board_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_arch_number = MACH_TYPE_VERSATILE_PB;
	gd->bd->bi_boot_params = CFG_BOOT_PARAMS;

	hisi_oem_init();

	hiusb_ohci_enable_clk();
	return 0;
}

int show_logo(void);

int misc_init_r (void)
{
#ifdef CONFIG_RANDOM_ETHADDR
	random_init_r();
#endif

#ifdef CFG_MMU_HANDLE
	mmu_pagedir_nocached_range(MEM_BASE_DDR + SZ_64K, MEM_BASE_DDR + SZ_64M - 0x700000, 0x700000);
#endif

#if defined(CONFIG_HI3560V200_LOGO)
	show_logo();
#endif
    extern int eth_set_host_mac_address(void);
    eth_set_host_mac_address();
	setenv("verify", "n");

#ifdef CONFIG_AUTO_UPDATE
	extern int do_auto_update(void);
#ifdef CFG_MMU_HANDLE
	dcache_stop();
#endif
	do_auto_update();
#ifdef CFG_MMU_HANDLE
	dcache_start();
#endif
#endif
	/*Add HUSS-E4V related init */
	e4v_init_i2c_led_chip();
	fan_force_start();
	buzzer_post_test(1);
	e4v_check_restore_factory();
	return (0);
}

int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = CFG_DDR_PHYS_OFFSET;
	gd->bd->bi_dram[0].size = CFG_DDR_SIZE;

	return 0;
}

#if defined(CONFIG_POST) || defined(CONFIG_LOGBUFFER)

void post_word_store (ulong a)
{
	return;
}

ulong post_word_load (void)
{
	return 0l;
}

/*
 * Returns 1 if keys pressed to start the power-on long-running tests
 * Called from board_init_f().
 */
int post_hotkeys_pressed(void)
{
	return (ctrlc());
}
#endif	/* CONFIG_POST || CONFIG_LOGBUFFER*/

